// Code generated by smithy-go-codegen DO NOT EDIT.

package types

import (
	smithydocument "github.com/aws/smithy-go/document"
	"time"
)

// An AWS account that is the administrator account of or a member of a behavior
// graph.
type Account struct {

	// The account identifier of the AWS account.
	//
	// This member is required.
	AccountId *string

	// The AWS account root user email address for the AWS account.
	//
	// This member is required.
	EmailAddress *string

	noSmithyDocumentSerde
}

// A behavior graph in Detective.
type Graph struct {

	// The ARN of the behavior graph.
	Arn *string

	// The date and time that the behavior graph was created. The value is in
	// milliseconds since the epoch.
	CreatedTime *time.Time

	noSmithyDocumentSerde
}

// Details about a member account that was invited to contribute to a behavior
// graph.
type MemberDetail struct {

	// The AWS account identifier for the member account.
	AccountId *string

	// The AWS account identifier of the administrator account for the behavior graph.
	AdministratorId *string

	// For member accounts with a status of ACCEPTED_BUT_DISABLED, the reason that the
	// member account is not enabled. The reason can have one of the following
	// values:
	//
	// * VOLUME_TOO_HIGH - Indicates that adding the member account would
	// cause the data volume for the behavior graph to be too high.
	//
	// * VOLUME_UNKNOWN -
	// Indicates that Detective is unable to verify the data volume for the member
	// account. This is usually because the member account is not enrolled in Amazon
	// GuardDuty.
	DisabledReason MemberDisabledReason

	// The AWS account root user email address for the member account.
	EmailAddress *string

	// The ARN of the behavior graph that the member account was invited to.
	GraphArn *string

	// The date and time that Detective sent the invitation to the member account. The
	// value is in milliseconds since the epoch.
	InvitedTime *time.Time

	// The AWS account identifier of the administrator account for the behavior graph.
	//
	// Deprecated: This property is deprecated. Use AdministratorId instead.
	MasterId *string

	// The member account data volume as a percentage of the maximum allowed data
	// volume. 0 indicates 0 percent, and 100 indicates 100 percent. Note that this is
	// not the percentage of the behavior graph data volume. For example, the data
	// volume for the behavior graph is 80 GB per day. The maximum data volume is 160
	// GB per day. If the data volume for the member account is 40 GB per day, then
	// PercentOfGraphUtilization is 25. It represents 25% of the maximum allowed data
	// volume.
	//
	// Deprecated: This property is deprecated. Use VolumeUsageInBytes instead.
	PercentOfGraphUtilization *float64

	// The date and time when the graph utilization percentage was last updated.
	//
	// Deprecated: This property is deprecated. Use VolumeUsageUpdatedTime instead.
	PercentOfGraphUtilizationUpdatedTime *time.Time

	// The current membership status of the member account. The status can have one of
	// the following values:
	//
	// * INVITED - Indicates that the member was sent an
	// invitation but has not yet responded.
	//
	// * VERIFICATION_IN_PROGRESS - Indicates
	// that Detective is verifying that the account identifier and email address
	// provided for the member account match. If they do match, then Detective sends
	// the invitation. If the email address and account identifier don't match, then
	// the member cannot be added to the behavior graph.
	//
	// * VERIFICATION_FAILED -
	// Indicates that the account and email address provided for the member account do
	// not match, and Detective did not send an invitation to the account.
	//
	// * ENABLED -
	// Indicates that the member account accepted the invitation to contribute to the
	// behavior graph.
	//
	// * ACCEPTED_BUT_DISABLED - Indicates that the member account
	// accepted the invitation but is prevented from contributing data to the behavior
	// graph. DisabledReason provides the reason why the member account is not
	// enabled.
	//
	// Member accounts that declined an invitation or that were removed from
	// the behavior graph are not included.
	Status MemberStatus

	// The date and time that the member account was last updated. The value is in
	// milliseconds since the epoch.
	UpdatedTime *time.Time

	// The data volume in bytes per day for the member account.
	VolumeUsageInBytes *int64

	// The data and time when the member account data volume was last updated.
	VolumeUsageUpdatedTime *time.Time

	noSmithyDocumentSerde
}

// A member account that was included in a request but for which the request could
// not be processed.
type UnprocessedAccount struct {

	// The AWS account identifier of the member account that was not processed.
	AccountId *string

	// The reason that the member account request could not be processed.
	Reason *string

	noSmithyDocumentSerde
}

type noSmithyDocumentSerde = smithydocument.NoSerde
