// Code generated by smithy-go-codegen DO NOT EDIT.

package types

import (
	smithydocument "github.com/aws/smithy-go/document"
	"time"
)

// The action that starts at the beginning of an incident. The response plan
// defines the action.
//
// The following types satisfy this interface:
//  ActionMemberSsmAutomation
type Action interface {
	isAction()
}

// The Systems Manager automation document to start as the runbook at the beginning
// of the incident.
type ActionMemberSsmAutomation struct {
	Value SsmAutomation

	noSmithyDocumentSerde
}

func (*ActionMemberSsmAutomation) isAction() {}

// Defines the Region and KMS key to add to the replication set.
type AddRegionAction struct {

	// The Region name to add to the replication set.
	//
	// This member is required.
	RegionName *string

	// The KMS key ID to use to encrypt your replication set.
	SseKmsKeyId *string

	noSmithyDocumentSerde
}

// Use the AttributeValueList to filter by string or integer values.
//
// The following types satisfy this interface:
//  AttributeValueListMemberStringValues
//  AttributeValueListMemberIntegerValues
type AttributeValueList interface {
	isAttributeValueList()
}

// The list of string values that the filter matches.
type AttributeValueListMemberStringValues struct {
	Value []string

	noSmithyDocumentSerde
}

func (*AttributeValueListMemberStringValues) isAttributeValueList() {}

// The list of integer values that the filter matches.
type AttributeValueListMemberIntegerValues struct {
	Value []int32

	noSmithyDocumentSerde
}

func (*AttributeValueListMemberIntegerValues) isAttributeValueList() {}

// The Systems Manager automation document process to start as the runbook at the
// beginning of the incident.
//
// The following types satisfy this interface:
//  AutomationExecutionMemberSsmExecutionArn
type AutomationExecution interface {
	isAutomationExecution()
}

// The Amazon Resource Name (ARN) of the automation process.
type AutomationExecutionMemberSsmExecutionArn struct {
	Value string

	noSmithyDocumentSerde
}

func (*AutomationExecutionMemberSsmExecutionArn) isAutomationExecution() {}

// The AWS Chatbot chat channel used for collaboration during an incident.
//
// The following types satisfy this interface:
//  ChatChannelMemberEmpty
//  ChatChannelMemberChatbotSns
type ChatChannel interface {
	isChatChannel()
}

// Used to remove the chat channel from an incident record or response plan.
type ChatChannelMemberEmpty struct {
	Value EmptyChatChannel

	noSmithyDocumentSerde
}

func (*ChatChannelMemberEmpty) isChatChannel() {}

// The SNS targets that AWS Chatbot uses to notify the chat channel of updates to
// an incident. You can also make updates to the incident through the chat channel
// by using the SNS topics.
type ChatChannelMemberChatbotSns struct {
	Value []string

	noSmithyDocumentSerde
}

func (*ChatChannelMemberChatbotSns) isChatChannel() {}

// A conditional statement with which to compare a value, after a timestamp, before
// a timestamp, or equal to a string or integer. If multiple conditions are
// specified, the conditionals become an ANDed statement. If multiple values are
// specified for a conditional, the values are ORd.
//
// The following types satisfy this interface:
//  ConditionMemberBefore
//  ConditionMemberAfter
//  ConditionMemberEquals
type Condition interface {
	isCondition()
}

// Before the specified timestamp
type ConditionMemberBefore struct {
	Value time.Time

	noSmithyDocumentSerde
}

func (*ConditionMemberBefore) isCondition() {}

// After the specified timestamp.
type ConditionMemberAfter struct {
	Value time.Time

	noSmithyDocumentSerde
}

func (*ConditionMemberAfter) isCondition() {}

// The value is equal to the provided string or integer.
type ConditionMemberEquals struct {
	Value AttributeValueList

	noSmithyDocumentSerde
}

func (*ConditionMemberEquals) isCondition() {}

// Defines the information about the Region you're deleting from your replication
// set.
type DeleteRegionAction struct {

	// The name of the Region you're deleting from the replication set.
	//
	// This member is required.
	RegionName *string

	noSmithyDocumentSerde
}

// Used to remove the chat channel from an incident record or response plan.
type EmptyChatChannel struct {
	noSmithyDocumentSerde
}

// Details about a timeline event during an incident.
type EventSummary struct {

	// The timeline event ID.
	//
	// This member is required.
	EventId *string

	// The time that the event occurred.
	//
	// This member is required.
	EventTime *time.Time

	// The type of event. The timeline event must be Custom Event.
	//
	// This member is required.
	EventType *string

	// The time that the timeline event was last updated.
	//
	// This member is required.
	EventUpdatedTime *time.Time

	// The Amazon Resource Name (ARN) of the incident that the event happened during.
	//
	// This member is required.
	IncidentRecordArn *string

	noSmithyDocumentSerde
}

// Filter the selection by using a condition.
type Filter struct {

	// The condition accepts before or after a specified time, equal to a string, or
	// equal to an integer.
	//
	// This member is required.
	Condition Condition

	// The key that you're filtering on.
	//
	// This member is required.
	Key *string

	noSmithyDocumentSerde
}

// The record of the incident that's created when an incident occurs.
type IncidentRecord struct {

	// The Amazon Resource Name (ARN) of the incident record.
	//
	// This member is required.
	Arn *string

	// The time that Incident Manager created the incident record.
	//
	// This member is required.
	CreationTime *time.Time

	// The string Incident Manager uses to prevent duplicate incidents from being
	// created by the same incident.
	//
	// This member is required.
	DedupeString *string

	// The impact of the incident on customers and applications.
	//
	// This member is required.
	Impact *int32

	// Details about the action that started the incident.
	//
	// This member is required.
	IncidentRecordSource *IncidentRecordSource

	// Who modified the incident most recently.
	//
	// This member is required.
	LastModifiedBy *string

	// The time at which the incident was most recently modified.
	//
	// This member is required.
	LastModifiedTime *time.Time

	// The current status of the incident.
	//
	// This member is required.
	Status IncidentRecordStatus

	// The title of the incident.
	//
	// This member is required.
	Title *string

	// The runbook, or automation document, that's run at the beginning of the
	// incident.
	AutomationExecutions []AutomationExecution

	// The chat channel used for collaboration during an incident.
	ChatChannel ChatChannel

	// The SNS targets that are notified when updates are made to an incident.
	NotificationTargets []NotificationTargetItem

	// The time at which the incident was resolved. This appears as a timeline event.
	ResolvedTime *time.Time

	// The summary of the incident. The summary is a brief synopsis of what occurred,
	// what is currently happening, and context.
	Summary *string

	noSmithyDocumentSerde
}

// Details about how the incident record was created and when.
type IncidentRecordSource struct {

	// The principal that started the incident.
	//
	// This member is required.
	CreatedBy *string

	// The service that started the incident. This can be manually created from
	// Incident Manager, automatically created using an AWS CloudWatch alarm, or Amazon
	// EventBridge event.
	//
	// This member is required.
	Source *string

	// The principal the assumed the role specified of the createdBy.
	InvokedBy *string

	// The resource that caused the incident to be created.
	ResourceArn *string

	noSmithyDocumentSerde
}

// Details describing an incident record.
type IncidentRecordSummary struct {

	// The Amazon Resource Name (ARN) of the incident.
	//
	// This member is required.
	Arn *string

	// The time the incident was created.
	//
	// This member is required.
	CreationTime *time.Time

	// Defines the impact to customers and applications.
	//
	// This member is required.
	Impact *int32

	// What caused Incident Manager to create the incident.
	//
	// This member is required.
	IncidentRecordSource *IncidentRecordSource

	// The current status of the incident.
	//
	// This member is required.
	Status IncidentRecordStatus

	// The title of the incident. This value is either provided by the response plan or
	// overwritten on creation.
	//
	// This member is required.
	Title *string

	// The time the incident was resolved.
	ResolvedTime *time.Time

	noSmithyDocumentSerde
}

// Basic details used in creating a response plan. The response plan is then used
// to create an incident record.
type IncidentTemplate struct {

	// The impact of the incident on your customers and applications.
	//
	// This member is required.
	Impact *int32

	// The title of the incident.
	//
	// This member is required.
	Title *string

	// Used to stop Incident Manager from creating multiple incident records for the
	// same incident.
	DedupeString *string

	// The SNS targets that are notified when updates are made to an incident.
	NotificationTargets []NotificationTargetItem

	// The summary of the incident. The summary is a brief synopsis of what occurred,
	// what's currently happening, and context.
	Summary *string

	noSmithyDocumentSerde
}

// Details and type of a related item.
type ItemIdentifier struct {

	// The type of related item. Incident Manager supports the following types:
	//
	// *
	// ANALYSIS
	//
	// * INCIDENT
	//
	// * METRIC
	//
	// * PARENT
	//
	// * ATTACHMENT
	//
	// * OTHER
	//
	// This member is required.
	Type ItemType

	// Details about the related item.
	//
	// This member is required.
	Value ItemValue

	noSmithyDocumentSerde
}

// Describes a related item.
//
// The following types satisfy this interface:
//  ItemValueMemberArn
//  ItemValueMemberUrl
//  ItemValueMemberMetricDefinition
type ItemValue interface {
	isItemValue()
}

// The Amazon Resource Name (ARN) of the related item, if the related item is an
// Amazon resource.
type ItemValueMemberArn struct {
	Value string

	noSmithyDocumentSerde
}

func (*ItemValueMemberArn) isItemValue() {}

// The URL, if the related item is a non-AWS resource.
type ItemValueMemberUrl struct {
	Value string

	noSmithyDocumentSerde
}

func (*ItemValueMemberUrl) isItemValue() {}

// The metric definition, if the related item is a metric in CloudWatch.
type ItemValueMemberMetricDefinition struct {
	Value string

	noSmithyDocumentSerde
}

func (*ItemValueMemberMetricDefinition) isItemValue() {}

// The SNS targets that are notified when updates are made to an incident.
//
// The following types satisfy this interface:
//  NotificationTargetItemMemberSnsTopicArn
type NotificationTargetItem interface {
	isNotificationTargetItem()
}

// The Amazon Resource Name (ARN) of the SNS topic.
type NotificationTargetItemMemberSnsTopicArn struct {
	Value string

	noSmithyDocumentSerde
}

func (*NotificationTargetItemMemberSnsTopicArn) isNotificationTargetItem() {}

// Information about a Region in your replication set.
type RegionInfo struct {

	// The status of the Region in the replication set.
	//
	// This member is required.
	Status RegionStatus

	// The most recent date and time that the Region's status was updated.
	//
	// This member is required.
	StatusUpdateDateTime *time.Time

	// The ID of the KMS key used to encrypt the data in this Region.
	SseKmsKeyId *string

	// Information displayed about the status of the Region.
	StatusMessage *string

	noSmithyDocumentSerde
}

// The mapping between a Region and the key that's used to encrypt the data.
type RegionMapInputValue struct {

	// The KMS key used to encrypt the data in your replication set.
	SseKmsKeyId *string

	noSmithyDocumentSerde
}

// Resources that responders use to triage and mitigate the incident.
type RelatedItem struct {

	// Details about the related item.
	//
	// This member is required.
	Identifier *ItemIdentifier

	// The title of the related item.
	Title *string

	noSmithyDocumentSerde
}

// Details about the related item you're adding.
//
// The following types satisfy this interface:
//  RelatedItemsUpdateMemberItemToAdd
//  RelatedItemsUpdateMemberItemToRemove
type RelatedItemsUpdate interface {
	isRelatedItemsUpdate()
}

// Details about the related item you're adding.
type RelatedItemsUpdateMemberItemToAdd struct {
	Value RelatedItem

	noSmithyDocumentSerde
}

func (*RelatedItemsUpdateMemberItemToAdd) isRelatedItemsUpdate() {}

// Details about the related item you're deleting.
type RelatedItemsUpdateMemberItemToRemove struct {
	Value ItemIdentifier

	noSmithyDocumentSerde
}

func (*RelatedItemsUpdateMemberItemToRemove) isRelatedItemsUpdate() {}

// The set of Regions that your Incident Manager data will be replicated to and the
// KMS key used to encrypt the data.
type ReplicationSet struct {

	// Details about who created the replication set.
	//
	// This member is required.
	CreatedBy *string

	// When the replication set was created.
	//
	// This member is required.
	CreatedTime *time.Time

	// Determines if the replication set deletion protection is enabled or not. If
	// deletion protection is enabled, you can't delete the last Region in the
	// replication set.
	//
	// This member is required.
	DeletionProtected *bool

	// Who last modified the replication set.
	//
	// This member is required.
	LastModifiedBy *string

	// When the replication set was last updated.
	//
	// This member is required.
	LastModifiedTime *time.Time

	// The map between each Region in your replication set and the KMS key that is used
	// to encrypt the data in that Region.
	//
	// This member is required.
	RegionMap map[string]RegionInfo

	// The status of the replication set. If the replication set is still pending, you
	// can't use Incident Manager functionality.
	//
	// This member is required.
	Status ReplicationSetStatus

	// The Amazon Resource Name (ARN) of the replication set.
	Arn *string

	noSmithyDocumentSerde
}

// The resource policy that allows Incident Manager to perform actions on resources
// on your behalf.
type ResourcePolicy struct {

	// The JSON blob that describes the policy.
	//
	// This member is required.
	PolicyDocument *string

	// The ID of the resource policy.
	//
	// This member is required.
	PolicyId *string

	// The Region that policy allows resources to be used in.
	//
	// This member is required.
	RamResourceShareRegion *string

	noSmithyDocumentSerde
}

// Details of the response plan that are used when creating an incident.
type ResponsePlanSummary struct {

	// The Amazon Resource Name (ARN) of the response plan.
	//
	// This member is required.
	Arn *string

	// The name of the response plan. This can't include spaces.
	//
	// This member is required.
	Name *string

	// The human readable name of the response plan. This can include spaces.
	DisplayName *string

	noSmithyDocumentSerde
}

// Details about the Systems Manager automation document that will be used as a
// runbook during an incident.
type SsmAutomation struct {

	// The automation document's name.
	//
	// This member is required.
	DocumentName *string

	// The Amazon Resource Name (ARN) of the role that the automation document will
	// assume when running commands.
	//
	// This member is required.
	RoleArn *string

	// The automation document's version to use when running.
	DocumentVersion *string

	// The key-value pair parameters to use when running the automation document.
	Parameters map[string][]string

	// The account that the automation document will be run in. This can be in either
	// the management account or an application account.
	TargetAccount SsmTargetAccount

	noSmithyDocumentSerde
}

// A significant event that happened during the incident.
type TimelineEvent struct {

	// A short description of the event.
	//
	// This member is required.
	EventData *string

	// The ID of the timeline event.
	//
	// This member is required.
	EventId *string

	// The time that the event occurred.
	//
	// This member is required.
	EventTime *time.Time

	// The type of event that occurred. Currently Incident Manager supports only the
	// Custom Event type.
	//
	// This member is required.
	EventType *string

	// The time that the timeline event was last updated.
	//
	// This member is required.
	EventUpdatedTime *time.Time

	// The Amazon Resource Name (ARN) of the incident that the event occurred during.
	//
	// This member is required.
	IncidentRecordArn *string

	noSmithyDocumentSerde
}

// Details about what caused the incident to be created in Incident Manager.
type TriggerDetails struct {

	// Identifies the service that sourced the event. All events sourced from within
	// AWS begin with "aws." Customer-generated events can have any value here, as long
	// as it doesn't begin with "aws." We recommend the use of Java package-name style
	// reverse domain-name strings.
	//
	// This member is required.
	Source *string

	// The time that the incident was detected.
	//
	// This member is required.
	Timestamp *time.Time

	// Raw data passed from either EventBridge, CloudWatch, or Incident Manager when an
	// incident is created.
	RawData *string

	// The ARN of the source that detected the incident.
	TriggerArn *string

	noSmithyDocumentSerde
}

// Details used when updating the replication set.
//
// The following types satisfy this interface:
//  UpdateReplicationSetActionMemberAddRegionAction
//  UpdateReplicationSetActionMemberDeleteRegionAction
type UpdateReplicationSetAction interface {
	isUpdateReplicationSetAction()
}

// Details about the Region that you're adding to the replication set.
type UpdateReplicationSetActionMemberAddRegionAction struct {
	Value AddRegionAction

	noSmithyDocumentSerde
}

func (*UpdateReplicationSetActionMemberAddRegionAction) isUpdateReplicationSetAction() {}

// Details about the Region that you're deleting to the replication set.
type UpdateReplicationSetActionMemberDeleteRegionAction struct {
	Value DeleteRegionAction

	noSmithyDocumentSerde
}

func (*UpdateReplicationSetActionMemberDeleteRegionAction) isUpdateReplicationSetAction() {}

type noSmithyDocumentSerde = smithydocument.NoSerde

// UnknownUnionMember is returned when a union member is returned over the wire,
// but has an unknown tag.
type UnknownUnionMember struct {
	Tag   string
	Value []byte

	noSmithyDocumentSerde
}

func (*UnknownUnionMember) isAction()                     {}
func (*UnknownUnionMember) isAttributeValueList()         {}
func (*UnknownUnionMember) isAutomationExecution()        {}
func (*UnknownUnionMember) isChatChannel()                {}
func (*UnknownUnionMember) isCondition()                  {}
func (*UnknownUnionMember) isItemValue()                  {}
func (*UnknownUnionMember) isNotificationTargetItem()     {}
func (*UnknownUnionMember) isRelatedItemsUpdate()         {}
func (*UnknownUnionMember) isUpdateReplicationSetAction() {}
